@@ -1,11 +1,9 @@
-#coding=utf-8
-
 import json
 import time
 import os
 import mimetypes
-import base64
 import datetime
+from websockets.sync.client import connect
 
 try:
 	from urllib import quote, urlpase
@@ -328,6 +326,7 @@ class BmobQuery:
 
 	def addWhereWithinGeoBox(self, key,southwest, northeast):
 		return self.putWhereFilter(key, {"$box": [southwest.__dict__, northeast.__dict__]}, "$within")
+	
 	# 列表
 	def addWhereContainedIn(self, key, value, isIn = True):
 		if isIn:
@@ -335,13 +334,25 @@ class BmobQuery:
 		else:
 			isIn = "$nin"
 		return self.putWhereFilter(key, value, isIn)
+	
 	def addWhereNotContainedIn(self, key, value):
 		return self.addWhereContainedIn(key, value, False)
+	
 	def addWhereContainsAll(self, key, value):
 		return self.putWhereFilter(key, value, "$all")
-	# 模糊查询
+	
 	def addWhereStrContains(self, key, value):
+		"""模糊查询（正则查询）
+
+		Args:
+			key (str): 列名
+			value (str): 是否存在的字符串
+
+		Returns:
+			self: 自身
+		"""		
 		return self.putWhereFilter(key, value, "$regex")
+	
 	# 子查询
 	def addWhereMatchesSelect(self, key, innerQuery, innerKey, innerTable = None, isMatch = True):
 		if isMatch:
@@ -351,6 +362,7 @@ class BmobQuery:
 		if isinstance(innerQuery, BmobQuery):
 			innerQuery = {"className": innerTable, "where": innerQuery.filter}
 		return self.putWhereFilter(key, {"key": innerKey, "query": innerQuery}, isMatch)
+	
 	def addWhereInQuery(self, key, value, className = None, isIn = True):
 		if isIn:
 			isIn = "$inQuery"
@@ -447,6 +459,8 @@ def BmobRequest(url, method = 'GET', headers = None, body = None, timeout = 10,
 class Bmob:
     
 	error = {}
+	chatList = {}
+	reply = ''
     
 	def __init__(self, application_id, rest_api_key):
 		"""初始化Bmob
@@ -459,7 +473,84 @@ class Bmob:
 		self.headers = {"X-Bmob-Application-Id": application_id, "X-Bmob-REST-API-Key": rest_api_key, "Content-Type": "application/json"}
 		self.appid = application_id
 		self.restkey = rest_api_key
-  
+
+	def connectAI(self):
+		"""初始化AI服务
+		"""		
+		url = 'wss://api.codenow.cn/1/ai'
+		self.ws = connect(url,additional_headers=self.headers)
+	
+	def closeAI(self):
+		"""关闭AI服务
+		"""		
+		try:
+			self.ws.close()
+		except Exception as e:
+			pass
+
+	def chat(self,chatmsg='你好',session='default'):
+		"""开始对话
+
+		Args:
+			chatmsg (str, optional): 发送的对话内容. Defaults to '你好'.
+			session (str, optional): 会话名称. Defaults to 'default'.
+
+		Returns:
+			str: AI服务器返回的文字
+		"""		
+		# 发送消息到AI服务器
+		if session not in self.chatList:
+			self.chatList[session] = []
+				
+		sessionchatlist = self.chatList[session]
+
+		if len(sessionchatlist) >=1:
+			last = sessionchatlist[-1]
+			if last['role'] =='assistant':
+				sessionchatlist.append({
+					'role':'user',
+					'content':chatmsg
+				})
+		else:
+			sessionchatlist.append({
+				'role':'user',
+				'content':chatmsg
+			})
+
+		if len(sessionchatlist) > 5:
+			realsend = sessionchatlist[-6:]
+		else:
+			realsend = sessionchatlist
+		
+		sendmsg = {
+			'messages':realsend,
+			'session':'python'
+		}
+
+		self.ws.send(json.dumps(sendmsg))
+		self.reply = ''
+
+		while True:
+			try:
+				message = self.ws.recv_messages.get()
+				data = json.loads(message)
+
+				if 'choices' in message:
+					finish = data['choices'][0]['finish_reason']
+					if finish!='':
+						sessionchatlist.append({
+							'role':'assistant',
+							'content':self.reply
+						})
+						return self.reply
+					self.reply += data['choices'][0]['delta']['content']		
+				else:
+					print('ai的数据格式错误')
+					return self.reply
+			except Exception as e:
+				print(e)
+				return self.reply
+
 	def getError(self):
 		"""获取错误信息
 
@@ -511,7 +602,8 @@ class Bmob:
 			'age':100
 		})
 		
-		其中，13800138002是注册的账号，123456是登录的密码，userInfo是其他的用户信息，可为空
+		其中，13800138002是注册的账号，123456是登录的密码，userInfo是其他的用户信息，可为空.
+
 		Args:
 			username (string): 账号
 			password (string): 密码
@@ -530,9 +622,9 @@ class Bmob:
 			userInfo['username'] = username
 			userInfo['password'] = password
 
-		return self._signUp(userInfo)
+		return self.__signUp(userInfo)
 
-	def _signUp(self, userInfo):
+	def __signUp(self, userInfo):
 		"""用户注册
 
 		Args:
@@ -867,7 +959,7 @@ class Bmob:
 		Returns:
 			int: 记录数。如果发生异常，返回None.
 		"""     
-		return self._basefindObjects(className,where=where,limit=0,count=1)
+		return self.__basefindObjects(className,where=where,limit=0,count=1)
 
 	def sum(self,className, sumName ,where = None):
 		"""查询where条件下，sumName的和
@@ -889,7 +981,7 @@ class Bmob:
 		Returns:
 			dict or int: 和的结果
 		"""     
-		return self._basefindObjects(className,where=where, sum=sumName)
+		return self.__basefindObjects(className,where=where, sum=sumName)
 
 	def max(self,className, maxName ,where = None):
 		"""查询where条件下，maxName列的最大值
@@ -910,7 +1002,7 @@ class Bmob:
 		Returns:
 			dict or int: 最大值
 		"""     
-		return self._basefindObjects(className,where=where, max=maxName)
+		return self.__basefindObjects(className,where=where, max=maxName)
 
 	def min(self,className, minName ,where = None):
 		"""查询where条件下，minName列的最小值
@@ -932,7 +1024,7 @@ class Bmob:
 		Returns:
 			dict or int: 最小值
 		"""     
-		return self._basefindObjects(className,where=where, min=minName)
+		return self.__basefindObjects(className,where=where, min=minName)
 
 	def mean(self,className, meanName ,where = None):
 		"""查询where条件下，meanName列的平均值
@@ -952,12 +1044,12 @@ class Bmob:
 		Returns:
 			dict or int: 平均值
 		"""     
-		return self._basefindObjects(className,where=where, average=meanName)
+		return self.__basefindObjects(className,where=where, average=meanName)
 
 	def findObjects(self, className, where = None, limit = None, skip = None, order = None, include = None, keys = None,groupby = None, groupcount = None, having = None):
-		return self._basefindObjects(className,where=where,limit=limit,skip=skip,order=order,include=include,keys=keys,groupby=groupby,groupcount=groupcount,having=having)
+		return self.__basefindObjects(className,where=where,limit=limit,skip=skip,order=order,include=include,keys=keys,groupby=groupby,groupcount=groupcount,having=having)
 	
-	def _basefindObjects(self, className, where = None, limit = None, skip = None, order = None, include = None, keys = None, count = None, groupby = None, groupcount = None, min = None, max = None, sum = None, average = None, having = None):
+	def __basefindObjects(self, className, where = None, limit = None, skip = None, order = None, include = None, keys = None, count = None, groupby = None, groupcount = None, min = None, max = None, sum = None, average = None, having = None):
 		"""多条数据的条件查询
 
 		示例代码：
@@ -1088,14 +1180,14 @@ class Bmob:
 		r = b.getObject("ai_log", "5949973169")
 		print(r.messages)
 
-		其中，ai_log是表名，5949973169是这条记录的objectId，messages是记录的字段名
+		其中，ai_log是表名，5949973169是这条记录的objectId，messages是记录的字段名.
 
 		Args:
-			className (string): 数据表名称
-			objectId (string): 这条记录的objectId
+			className (string): 数据表名称.
+			objectId (string): 这条记录的objectId.
 
 		Returns:
-			object: 数据的查询结果，
+			object: 数据的查询结果.
 		"""	
 		
 		bmobResult =  BmobRequest(url = self.domain + '/1/classes/' + className + '/' + objectId, method = 'GET', headers = self.headers)
@@ -1146,7 +1238,15 @@ class Bmob:
 			return None
 
 	def delFile(self,url):
+		"""删除文件
+
+		Args:
+			url (str): 要删除的文件的url
+
+		Returns:
+			bool: 是否删除成功.
+		"""		
 		path = urlparse(url).path
-		bmobResult =  BmobRequest(url = self.domain + f'/2/files/upyun/{path}', method = 'DELETE', headers = self.headers)
+		bmobResult =  BmobRequest(url = self.domain + f'/2/files/upyun{path}', method = 'DELETE', headers = self.headers)
 		self.error = bmobResult.getError()
 		return dealBmobResult(bmobResult,'删除文件')
\ No newline at end of file
